package cz.mendelu.dnaAnalyser.analyse.palindrome.stability;

import cz.mendelu.dnaAnalyser.analyse.palindrome.model.PalindromeModel;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class NNModelStabilityCalculator implements StabilityCalculator {

    private static final Map<String, Double> table = new HashMap<>();
    static {
        table.put("AA/TT", -1.00);
        table.put("AT/TA", -0.88);
        table.put("AC/TG", -1.45);
        table.put("AG/TC", -1.00);

        table.put("TA/AT", -0.58);
        table.put("TT/AA", -1.00);
        table.put("TC/AG", -1.00);
        table.put("TG/AC", -1.45);

        table.put("CA/GT", -1.45);
        table.put("CT/GA", -1.28);
        table.put("CC/GG", -1.84);
        table.put("CG/GC", -2.17);

        table.put("GA/CT", -1.30);
        table.put("GT/CA", -1.44);
        table.put("GC/CG", -2.24);
        table.put("GG/CC", -1.84);

        table.put("G-C", 0.98);
        table.put("C-G", 0.98);
        table.put("A-T", 1.03);
        table.put("T-A", 1.03);
    }

    @Override
    public double calculateCruciForm(PalindromeModel palindromeModel) {
        return calculate(palindromeModel.iterator());
    }

    @Override
    public double calculateLinearForm(PalindromeModel palindromeModel) {
        return calculate(palindromeModel.linearFormIterator());
    }

    private double calculate(Iterator<PalindromeModel.Entry> i) {
        PalindromeModel.Entry first = i.next();
        PalindromeModel.Entry second;

        double sum = table.getOrDefault(makeKey(first), 0.0);
        while (i.hasNext()) {
            second = i.next();
            String key = makeKey(first, second);
            double value = table.getOrDefault(key, 0.0);
            sum += value;
            first = second;
        }
        sum += table.getOrDefault(makeKey(first), 0.0);
        return sum;
    }

    private String makeKey(PalindromeModel.Entry first, PalindromeModel.Entry second) {
        return first.getNuclid().name()
                + second.getNuclid().name()
                + "/"
                + first.getOposite().name()
                + second.getOposite().name();
    }

    private String makeKey(PalindromeModel.Entry entry) {
        return entry.getNuclid().name()
                + "-"
                + entry.getOposite().name();
    }
}
