package cz.mendelu.dnaAnalyser.sequence;

import com.fasterxml.jackson.annotation.JsonIgnore;
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide;
import cz.mendelu.dnaAnalyser.user.User;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import cz.mendelu.dnaAnalyser.utils.model.Owned;
import cz.mendelu.dnaAnalyser.utils.model.Tagged;
import lombok.*;
import org.hibernate.annotations.CreationTimestamp;

import javax.persistence.*;
import java.util.*;

/**
 * Created by Jan Kolomazník on 25.6.17.
 */

@Data
@Entity
@Builder
@NoArgsConstructor(access = AccessLevel.PROTECTED)
@AllArgsConstructor(access = AccessLevel.PROTECTED)
@Table(name = "sequences")
public class Sequence implements Identifiable, Owned, Tagged {

    @Id
    @Builder.Default
    private UUID id = UUID.randomUUID();

    @JsonIgnore
    private UUID bufferId;

    private String name;

    @Temporal(TemporalType.TIMESTAMP)
    @CreationTimestamp
    private Date created;

    private SequenceType type;

    private Boolean circular;

    private Integer length;

    private String ncbi;

    @ElementCollection
    private Set<String> tags;

    @ManyToOne(cascade = CascadeType.DETACH)
    @JsonIgnore
    private User owner;

    private String fastaComment;

    @ElementCollection
    private Map<Nucleotide, Integer> nucleicCounts;

    public Map<Nucleotide, Integer> getNucleicCounts() {
        return (nucleicCounts == null || nucleicCounts.isEmpty())
                ? null
                : Collections.unmodifiableMap(nucleicCounts);
    }
}
