package cz.mendelu.dnaAnalyser.exporter

import cz.mendelu.dnaAnalyser.sequence.Sequence;
import cz.mendelu.dnaAnalyser.sequence.SequenceService;
import cz.mendelu.dnaAnalyser.utils.analyse.repository.AnalyseRepository;
import cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex.QuadruplexDataSessionService;
import cz.mendelu.dnaAnalyser.analyse.rloopr.RloopDataSessionService;
import cz.mendelu.dnaAnalyser.utils.analyse.data.ExporterEnums.Analysis;
import cz.mendelu.dnaAnalyser.utils.analyse.data.ExporterEnums.Output;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import com.opencsv.CSVWriter;
import com.opencsv.ResultSetHelperService;
import java.io.StringWriter;

import spock.lang.Specification
import spock.lang.Ignore

class ExporterServiceTest extends Specification {

    ExporterService exporter = new ExporterService()

    SequenceService sequenceService
    QuadruplexDataSessionService quadruplexDataSessionService
    RloopDataSessionService rloopDataSessionService
    AnalyseRepository analyseRepository

    UUID uuid
    Sequence sequence = new Sequence(id: uuid)
    ResultSet mockedResultSet
    Statement mockedStatement
    Connection mockedConn

    def setup() {
        exporter.analyseRepository = analyseRepository = Mock(AnalyseRepository)
        exporter.sequenceService = sequenceService = Mock(SequenceService)
        exporter.quadruplexDataSessionService = quadruplexDataSessionService = Mock(QuadruplexDataSessionService)
        exporter.rloopDataSessionService = rloopDataSessionService = Mock(RloopDataSessionService)
        
        uuid = UUID.randomUUID()

        mockedResultSet = Mock(ResultSet)
        mockedStatement = Mock(Statement)
        mockedConn = Mock(Connection)

        mockedResultSet.next() >> false
        exporter.transforResultSetToCsv(mockedResultSet) >> ""
    }

    def "test aggregate to CSV export"() {
        given:
            def id = UUID.randomUUID()
            def analysis = Analysis.AGGREGATE
            def format = Output.CSV

        when:
            exporter.exportData(id, analysis, format)
        then:
            1 * exporter.quadruplexDataSessionService.openConnection(id) >> mockedConn
            1 * mockedConn.createStatement() >> mockedStatement
            1 * mockedStatement.executeQuery('SELECT * FROM AGGREGATE')
            1 * mockedStatement.close()
            1 * mockedConn.close()
    }

    def "test quadruplex to CSV export"() {
        given:
            def id = UUID.randomUUID()
            def analysis = Analysis.QUADRUPLEX
            def format = Output.CSV

        when:
            exporter.exportData(id, analysis, format)
        then:
            1 * exporter.quadruplexDataSessionService.openConnection(id) >> mockedConn
            1 * mockedConn.createStatement() >> mockedStatement
            1 * mockedStatement.executeQuery('SELECT * FROM QUADRUPLEX')
            1 * mockedStatement.close()
            1 * mockedConn.close()
    }

    def "test rloop to CSV export"() {
        given:
            def id = UUID.randomUUID()
            def analysis = Analysis.RLOOP
            def format = Output.CSV

        when:
            exporter.exportData(id, analysis, format)
        then:
            1 * exporter.rloopDataSessionService.openConnection(id) >> mockedConn
            1 * mockedConn.createStatement() >> mockedStatement
            1 * mockedStatement.executeQuery('SELECT * FROM RLOOP')
            1 * mockedStatement.close()
            1 * mockedConn.close()
    }

    @Ignore
    def "test quadruplex to bedgraph export"() {
        given:
            def id = UUID.randomUUID()
            def analysis = Analysis.QUADRUPLEX
            def format = Output.BEDGRAPH

        when:
            exporter.exportData(id, analysis, format)
        then:
            1 * exporter.quadruplexDataSessionService.openConnection(id) >> mockedConn
            1 * mockedConn.createStatement() >> mockedStatement
            1 * mockedStatement.executeQuery('SELECT * FROM QUADRUPLEX') >> mockedResultSet
            1 * mockedStatement.close()
            1 * mockedConn.close()
            1 * analyseRepository.getSequence(id)
    }

    def "test ChromName"() {
        given:
            def id = UUID.randomUUID()
        when:
            exporter.getChromName(id)
        then:
            1 * analyseRepository.getSequence(id) >> sequence
            1 * sequenceService.getName(null)
            1 * sequenceService.getNcbi(null)
    }

}
