package cz.mendelu.dnaAnalyser.sequence.annotation.services

import cz.mendelu.dnaAnalyser.sequence.NCBISequenceService
import cz.mendelu.dnaAnalyser.sequence.annotation.Annotation
import cz.mendelu.dnaAnalyser.sequence.annotation.AnnotationRepository
import cz.mendelu.dnaAnalyser.sequence.annotation.AnnotationService
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.Feature
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.FeatureDataSessionService
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.FeatureMapper
import cz.mendelu.dnaAnalyser.utils.controller.PaginationParam
import org.apache.ibatis.session.SqlSession
import org.springframework.data.domain.Pageable
import spock.lang.Specification

import java.util.stream.Stream



class AnnotationServiceTest extends Specification {

    AnnotationService annotationService
    AnnotationRepository annotationRepository;
    byte[] buffer
    String tmpDir
    FeatureDataSessionService featureDataSessionService


    def setup() {
        annotationService = new AnnotationService()
        annotationService.ncbiSequenceService = Mock(NCBISequenceService)
        annotationService.annotationRepository = annotationRepository = Mock(AnnotationRepository)

        buffer = '103614\t107842\tmobile_element'.bytes
        tmpDir = File.createTempDir().getPath()
        featureDataSessionService = new FeatureDataSessionService(
                datasourceUrl: "jdbc:h2:file:$tmpDir/analyse-id;DB_CLOSE_ON_EXIT=FALSE",
                datasourceDriverClassName: 'org.h2.Driver',
                datasourceUsername: '',
        )
        annotationService.featureDataSessionService = featureDataSessionService

    }


    def "ParseAnnotation"() {
        setup:
        def NCBI = "NC_004354.4";
        def annotation = new Annotation(
                id: UUID.fromString("090e5461-6dd3-44ff-8e01-7b84435857fa"),
                start: 103600,
                stop: 107900,
                type: "mobile_element"
        )
        annotation.setStart(103600)
        annotation.setStop(107900)
        annotation.setType("mobile_element")


        def type = "mobile_element"
        def start = 103600
        def stop = 107900
        def session = featureDataSessionService.openDataSession(annotation);


        Map<String, Stream<String>> downloadQue = new HashMap<>()
        downloadQue.put(NCBI, new BufferedReader(new InputStreamReader(new ByteArrayInputStream(buffer))).lines())
        NCBISequenceService ncbiStub = Stub()
        ncbiStub.downloadFeature(_ as String) >> { String ncbiId -> return downloadQue.getOrDefault(ncbiId, null) }
        annotationService.ncbiSequenceService = ncbiStub

        when:
        def result = annotationService.parseAnnotation(NCBI, type, start, stop, annotation, session)
        then:
        result[0].get("positions")[0] == "103614"
        result[0].get("positions")[1] == "107842"
        result[0].get("type") == "mobile_element"
    }


    def "GetFeatures"(){
        setup:
        def annotation = new Annotation(
                id: UUID.fromString("090e5461-6dd3-44ff-8e01-7b84435857fa")
        )
        annotation.setStart(103600)
        annotation.setStop(107900)
        annotation.setType("mobile_element")

        SqlSession sqlSession = featureDataSessionService.openDataSession(annotation);
        FeatureMapper featureMapper = sqlSession.getMapper(FeatureMapper.class);
        def feature = new Feature(null, "mobile_element", "103614", "107842", annotation.getId().toString());
        featureMapper.insert(feature);
        sqlSession.commit();

        when:
        def result = annotationService.getFeatures(annotation);

        then:
        result[0].startPosition == "103614"
        result[0].endPosition == "107842"
        result[0].featureType == "mobile_element"

    }

    def "RetrieveFeatures"(){
        setup:
        def feature = new Feature()
/*                featureType: "gene",
                startPosition: 10,
                endPosition: 10,
                areaSize: 100
        )*/
        feature.setStartPosition("103600")
        feature.setEndPosition("107900")
        feature.setFeatureType("mobile_element")

        PaginationParam param = new PaginationParam();
        Pageable pageable = param.toPageable(null);

        when:
        def result = annotationService.retrieveFeatures(Collections.singletonList(feature), pageable);

        then:
        result[0].type == "mobile_element"

    }

}
