package cz.mendelu.dnaAnalyser.utils.nio

import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide
import cz.mendelu.dnaAnalyser.sequence.nucleotide.NucleotideBuffer
import spock.lang.Ignore
import spock.lang.Specification

import java.nio.ByteBuffer

class NucleotideBufferTest extends Specification {

    NucleotideBuffer nucleotideBuffer
    ByteBuffer byteBuffer

    def setup() {
        byteBuffer = Mock(ByteBuffer)
        nucleotideBuffer = NucleotideBuffer.wrap(byteBuffer)
    }

    def "wrap ByteBuffer"() {
        when:
        def decoratesBuffer = NucleotideBuffer.wrap(byteBuffer)

        then:
        decoratesBuffer.buffer == byteBuffer
    }

    def "slice()"() {
        when:
        def result = nucleotideBuffer.slice()
        then:
        result != nucleotideBuffer
        1 * byteBuffer.slice() >> byteBuffer
    }

    def "duplicate()"() {
        when:
        def result = nucleotideBuffer.duplicate()
        then:
        result == nucleotideBuffer
        1 * byteBuffer.duplicate() >> byteBuffer
    }

    def "get()"() {
        when:
        def result = nucleotideBuffer.get()
        then:
        result == Nucleotide.N
        1 * byteBuffer.get() >> Nucleotide.N.toByte()
    }

    def "get(index)"() {
        when:
        def result = nucleotideBuffer.get(1)
        then:
        result == Nucleotide.N
        1 * byteBuffer.get(1) >> Nucleotide.N.toByte()
    }

    @Ignore
    def "capacity()"() {
        when:
        def result = nucleotideBuffer.getCapacity()
        then:
        result == 123
        1 * byteBuffer.capacity() >> 123

    }

    @Ignore
    def "getPosition()"() {
        when:
        def result = nucleotideBuffer.getPosition()
        then:
        result == 159
        1 * byteBuffer.position() >> 159
    }

    @Ignore
    def "setPosition(1)"() {
        when:
        nucleotideBuffer.setPosition(1)
        then:
        1 * byteBuffer.position(1) >> byteBuffer
    }

    @Ignore
    def "getLimit()"() {
        when:
        def result = nucleotideBuffer.getLimit()
        then:
        result == 753
        1 * byteBuffer.limit() >> 753
    }

    @Ignore
    def "setLimit(1)"() {
        when:
        nucleotideBuffer.setLimit(1)
        then:
        1 * byteBuffer.limit(1)
    }

    @Ignore
    def "mark()"() {
        when:
        def result = nucleotideBuffer.mark()
        then:
        result == nucleotideBuffer
        1 * byteBuffer.mark() >> byteBuffer
    }

    @Ignore
    def "reset()"() {
        when:
        def result = nucleotideBuffer.reset()
        then:
        result == nucleotideBuffer
        1 * byteBuffer.reset() >> byteBuffer
    }

    @Ignore
    def "clear()"() {
        when:
        def result = nucleotideBuffer.clear()
        then:
        result == nucleotideBuffer
        1 * byteBuffer.clear() >> byteBuffer
    }

    @Ignore
    def "flip()"() {
        when:
        def result = nucleotideBuffer.flip()
        then:
        result == nucleotideBuffer
        1 * byteBuffer.flip() >> byteBuffer
    }

    @Ignore
    def "rewind()"() {
        when:
        def result = nucleotideBuffer.rewind()
        then:
        result == nucleotideBuffer
        1 * byteBuffer.rewind() >> byteBuffer
    }

    @Ignore
    def "remaining()"() {
        when:
        def result = nucleotideBuffer.remaining()
        then:
        result == 951
        1 * byteBuffer.remaining() >> 951
    }

    @Ignore
    def "hasRemaining()"() {
        when:
        def result = nucleotideBuffer.hasRemaining()
        then:
        result == true
        1 * byteBuffer.hasRemaining() >> true
    }
}
